@@ -16,6 +16,11 @@
 import mock
 import pytest
 
+try:
+    import grpc  # noqa: F401
+except ImportError:
+    pytest.skip("No GRPC", allow_module_level=True)
+
 from google.api_core import exceptions
 from google.api_core import operation_async
 from google.api_core import operations_v1
@@ -153,7 +158,7 @@ async def test_exception():
 
 @mock.patch("asyncio.sleep", autospec=True)
 @pytest.mark.asyncio
-async def test_unexpected_result(unused_sleep):
+async def test_done_with_no_error_or_response(unused_sleep):
     responses = [
         make_operation_proto(),
         # Second operation response is done, but has not error or response.
@@ -161,9 +166,9 @@ async def test_unexpected_result(unused_sleep):
     ]
     future, _, _ = make_operation_future(responses)
 
-    exception = await future.exception()
+    result = await future.result()
 
-    assert "Unexpected state" in "{!r}".format(exception)
+    assert isinstance(result, struct_pb2.Struct)
 
 
 def test_from_gapic():
@@ -177,15 +182,15 @@ def test_from_gapic():
         operations_client,
         struct_pb2.Struct,
         metadata_type=struct_pb2.Struct,
-        grpc_metadata=[('x-goog-request-params', 'foo')]
+        grpc_metadata=[("x-goog-request-params", "foo")],
     )
 
     assert future._result_type == struct_pb2.Struct
     assert future._metadata_type == struct_pb2.Struct
     assert future.operation.name == TEST_OPERATION_NAME
     assert future.done
-    assert future._refresh.keywords["metadata"] == [('x-goog-request-params', 'foo')]
-    assert future._cancel.keywords["metadata"] == [('x-goog-request-params', 'foo')]
+    assert future._refresh.keywords["metadata"] == [("x-goog-request-params", "foo")]
+    assert future._cancel.keywords["metadata"] == [("x-goog-request-params", "foo")]
 
 
 def test_deserialize():
