@@ -14,6 +14,12 @@
 
 
 import mock
+import pytest
+
+try:
+    import grpc  # noqa: F401
+except ImportError:
+    pytest.skip("No GRPC", allow_module_level=True)
 
 from google.api_core import exceptions
 from google.api_core import operation
@@ -163,7 +169,7 @@ def test_exception_with_error_code():
     assert isinstance(exception, exceptions.NotFound)
 
 
-def test_unexpected_result():
+def test_done_with_no_error_or_response():
     responses = [
         make_operation_proto(),
         # Second operation response is done, but has not error or response.
@@ -171,9 +177,7 @@ def test_unexpected_result():
     ]
     future, _, _ = make_operation_future(responses)
 
-    exception = future.exception()
-
-    assert "Unexpected state" in "{!r}".format(exception)
+    assert isinstance(future.result(), struct_pb2.Struct)
 
 
 def test__refresh_http():
